import typing as t

import click
from click.core import Parameter, Context


class TimeParamType(click.ParamType):
    """Convert a duration as a string or integer to a number of seconds.

    If an integer is provided it is treated as seconds and is unchanged.

    String durations can have a suffix of 's', 'm', 'h', 'd', 'w', or 'y'.
    No suffix is treated as seconds.
    """
    name = 'time duration'

    def convert(self, value: t.Any, param: t.Optional['Parameter'], ctx: t.Optional['Context']) -> int:
        if isinstance(value, int):
            return value

        if isinstance(value, str) and value.isdigit():
            return int(value)

        second = 1
        minute = 60 * second
        hour = 60 * minute
        day = 24 * hour
        week = 7 * day
        year = 365 * day
        sizes = {'s': second, 'm': minute, 'h': hour, 'd': day, 'w': week, 'y': year}
        suffix = value[-1]

        if suffix in sizes:
            value = value[:-1]
            size = sizes[suffix]
        else:
            self.fail(f'{value!r} is not a valid time duration expression', param, ctx)
        return int(value) * size


TIME = TimeParamType()



from typing import Union

import yaml
from tabulate import tabulate


class IndentedListDumper(yaml.SafeDumper):
    def increase_indent(self, flow=False, indentless=False):
        return super(IndentedListDumper, self).increase_indent(flow, False)


def pretty_table(data: list, headers: Union[list, None] = None, tablefmt: str = 'github') -> str:
    """Tabulate wrapper function with predefined format"""
    if headers is None:
        table = tabulate(data, tablefmt=tablefmt)
    else:
        table = tabulate(data, headers=headers, tablefmt=tablefmt)
    return table


def pretty_info(data: dict) -> str:
    """Print data in a nice form"""
    return str(yaml.safe_dump(data))



import click
import yaml
from ecsclient.common.exceptions import ECSClientException

from ..._util.echo import success
from ..._util.exceptions import EcsmgmtClickException
from ..._util.format import IndentedListDumper


@click.command()
@click.argument('bucket-name', type=click.STRING)
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, bucket_name, namespace):
    """Set bucket acl

    **This command is only for advanced users!**
    """
    client = obj['client']

    acl_get = client.bucket.get_acl(bucket_name=bucket_name, namespace=namespace)
    acl_before = acl_get['acl']
    # removing to supported values
    acl_before.pop('default_group_dir_perms')
    acl_before.pop('default_group_file_perms')

    info = """---
# Comments get ignored, save & exit to commit changes
# Reference:
# owner: The name of bucket owner
# default_group: The default group of the bucket
#
# A collection of users and their corresponding permissions
# user_acl:
#   - permission:
#       - full_control
#     user: myuser1
#
# A collection of groups and their corresponding permissions
# group_acl:
#   - permission':
#       - read
#     group: public
#
# A collection of custom groups and their corresponding permissions
# customgroup_acl:
#   - permission:
#       - delete
#       - read
#       - write
#     customgroup: cgroup1

"""

    acl_before_yml = yaml.dump(acl_before, Dumper=IndentedListDumper)
    acl_new = click.edit(f'{info}{acl_before_yml}', extension='yml', require_save=True)

    if acl_new is None:
        click.echo('No changes detected (editor did not save)')
        return

    acl_new_loaded = yaml.safe_load(acl_new)

    if acl_new_loaded == acl_before:
        click.echo('No changes detected')
        return

    try:
        client.bucket.set_acl(bucket_name=bucket_name, namespace=namespace, **acl_new_loaded)
        success(f'Set acls for bucket "{bucket_name}" to:')
        click.echo(yaml.dump(acl_new_loaded, Dumper=IndentedListDumper))
    except ECSClientException as e:
        raise EcsmgmtClickException(e.message)



import click

from .._util.format import pretty_table


@click.command()
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, namespace):
    """List existing users in namespace
    """
    client = obj['client']

    user_request = client.object_user.list(namespace=namespace)
    user_list = [(entry['namespace'], entry['userid']) for entry in user_request['blobuser']]
    headers = ['Namespace', 'User ID']
    table = pretty_table(user_list, headers)
    click.echo(table)



import click
from ecsclient.common.exceptions import ECSClientException

from .._util.echo import success
from .._util.exceptions import EcsmgmtClickException


@click.command()
@click.argument('bucket-name', type=click.STRING)
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, bucket_name, namespace):
    """Create bucket
    """
    client = obj['client']

    if len(bucket_name) > 255:
        raise EcsmgmtClickException('Bucket name too long. Maximum allowed length is 255 chars.')
    if not all(x.isalnum() or x == '.' or x == '-' or x == '_' for x in bucket_name):
        raise EcsmgmtClickException('Bucket name contains invalid characters. Only alphanumeric, ".", "-" or "_" is allowed.')

    try:
        client.bucket.create(bucket_name=bucket_name, namespace=namespace)
        success(f'created bucket "{bucket_name}" in namespace "{namespace}"')
    except ECSClientException as e:
        raise EcsmgmtClickException(e.message)



import click

from ..._util.format import pretty_info


@click.command()
@click.argument('bucket-name', type=click.STRING)
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, bucket_name, namespace):
    """Get bucket acl
    """
    client = obj['client']

    res = client.bucket.get_acl(bucket_name=bucket_name, namespace=namespace)

    click.secho(f'Bucket "{bucket_name}" acl:', bold=True)
    click.echo(pretty_info(res))



import click

from ._util.format import pretty_info


@click.command()
@click.pass_obj
def cli(obj):
    """Get acl permissions
    """
    client = obj['client']

    res = client.bucket.get_acl_permissions()

    click.secho('Available acl permissions:', bold=True)
    click.echo(pretty_info(res))
